Feature: tag filters
  In order to test application behavior
  As a test suite
  I need to be able to filter features and scenarios by tags

  Scenario: should filter outline examples by tags
    Given a feature "normal.feature" file:
      """
      Feature: outline
      
        Background:
          Given passing step
          And passing step without return
      
        Scenario Outline: parse a scenario
          Given a feature path "<path>"
          When I parse features
          Then I should have <num> scenario registered
      
          Examples:
            | path                    | num |
            | features/load.feature:3 | 0   |
      
          @used
          Examples:
            | path                    | num |
            | features/load.feature:6 | 1   |
      """
    When I run feature suite with tags "@used"
    Then the suite should have passed
    And the following steps should be passed:
      """
      I parse features
      a feature path "features/load.feature:6"
      I should have 1 scenario registered
      """
    And I should have 1 scenario registered

  Scenario: should filter scenarios by X tag
    Given a feature "normal.feature" file:
      """
      Feature: tagged
      
        @x
        Scenario: one
          Given a feature path "one"
      
        @x
        Scenario: two
          Given a feature path "two"
      
        @x @y
        Scenario: three
          Given a feature path "three"
      
        @y
        Scenario: four
          Given a feature path "four"
      """
    When I run feature suite with tags "@x"
    Then the suite should have passed
    And I should have 3 scenario registered
    And the following steps should be passed:
      """
      a feature path "one"
      a feature path "two"
      a feature path "three"
      """

  Scenario: should filter scenarios by X tag not having Y
    Given a feature "normal.feature" file:
      """
      Feature: tagged
      
        @x
        Scenario: one
          Given a feature path "one"
      
        @x
        Scenario: two
          Given a feature path "two"
      
        @x @y
        Scenario: three
          Given a feature path "three"
      
        @y @z
        Scenario: four
          Given a feature path "four"
      """
    When I run feature suite with tags "@x && ~@y"
    Then the suite should have passed
    And I should have 2 scenario registered
    And the following steps should be passed:
      """
      a feature path "one"
      a feature path "two"
      """

  Scenario: should filter scenarios having Y and Z tags
    Given a feature "normal.feature" file:
      """
      Feature: tagged
      
        @x
        Scenario: one
          Given a feature path "one"
      
        @x
        Scenario: two
          Given a feature path "two"
      
        @x @y
        Scenario: three
          Given a feature path "three"
      
        @y @z
        Scenario: four
          Given a feature path "four"
      """
    When I run feature suite with tags "@y && @z"
    Then the suite should have passed
    And I should have 1 scenario registered
    And the following steps should be passed:
      """
      a feature path "four"
      """

  Scenario: empty filter and scenarios with f-tag - are executed only
    Given a feature "normal.feature" file:
      """
      Feature: f-tagged
      
        Scenario: one
          Given a feature path "one"
      
        Scenario: two
          Given a feature path "two"
      
        @f
        Scenario: three
          Given a feature path "three"
      
        @f
        Scenario: four
          Given a feature path "four"
      """
    When I run feature suite with tags ""
    Then the suite should have passed
    And I should have 2 scenario registered
    And the following steps should be passed:
      """
      a feature path "three"
      a feature path "four"
      """

  Scenario: two feature files and scenarios with f-tag - are executed only
    Given a feature "normal.feature" file:
      """
      Feature: f-tagged
      
        Scenario: one
          Given a feature path "one"
      
        Scenario: two
          Given a feature path "two"
      
        @f
        Scenario: three
          Given a feature path "three"
      
        @f
        Scenario: four
          Given a feature path "four"
      """
    And a feature "another.feature" file:
      """
      Feature: non-tagged
      
        Scenario: five
          Given a feature path "five"
      
        Scenario: six
          Given a feature path "six"
      
        Scenario: seven
          Given a feature path "seven"
      
        Scenario: eight
          Given a feature path "eight"
      """
    When I run feature suite with tags ""
    Then the suite should have passed
    And I should have 2 scenario registered
    And the following steps should be passed:
      """
      a feature path "three"
      a feature path "four"
      """

  Scenario: empty filter and scenarios with x-tag - are skipped
    Given a feature "normal.feature" file:
      """
      Feature: x-tagged
      
        Scenario: one
          Given a feature path "one"
      
        Scenario: two
          Given a feature path "two"
      
        @x
        Scenario: three
          Given a feature path "three"
      
        @x
        Scenario: four
          Given a feature path "four"
      """
    When I run feature suite with tags ""
    Then the suite should have passed
    And I should have 2 scenario registered
    And the following steps should be passed:
      """
      a feature path "one"
      a feature path "two"
      """
